import pyvisa


class Keithley6510:
    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)

    def reset(self):
        # Reset the instrument
        self.pyvisa.write("*RST")

    def setup_measurement(self):
        # # Set up the measurement configuration
        # self.daq.write(":SENS:FUNC 'RES'")
        # self.daq.write(":SENS:RES:MODE MAN")  # Manual range mode
        # self.daq.write(":SENS:RES:RANG 100")  # Set the range to 100 Ohms
        # self.daq.write(":SENS:RES:NPLC 1")    # Set integration rate to 1 PLC (Power Line Cycles)
        # Configure the instrument
        self.pyvisa.write(":SENS:FUNC 'FRES'")  # Set the function to measure resistance
        self.pyvisa.write(":SENS:FRES:NPLC 10")  # Set integration time to 10 PLC
        self.pyvisa.write(":SENS:FRES:RANG:AUTO ON")  # Enable auto-range
        self.pyvisa.write(":SENS:FRES:OCOM ON")  # Enable offset compensation

    def read_resistance(self):
        # Read the measured resistance value
        resistance = self.pyvisa.query(":MEAS:FRES?")
        return float(resistance)

    def close(self):
        # Close the connection
        self.daq.close()



import csv
import sys

# import schedule
import time
from datetime import datetime
from pathlib import Path

import mariadb
from numpy import array

sys.path.append(r"Q:\qnnpy")
import qnnpy.functions.functions as qf

PATH = "S:/SC/InstrumentLogging/Cryogenics/Ice/ice-log/Results/"


def load_data_to_database(filename: str, table_name: str, connection=None):
    if connection is not None:
        conn = connection
    else:
        conn = qf.database_connection()

    cur = conn.cursor()
    command = (
        "LOAD DATA LOCAL INFILE '%s' IGNORE INTO TABLE `%s` FIELDS TERMINATED BY ',' LINES TERMINATED BY '\n' "
        % (filename, table_name)
    )
    cur.execute(command)
    conn.commit()
    time.sleep(2)
    conn.close()


def log_many_rows_to_database(table_name: str, connection=None, **kwargs):
    """
    table_name [str]: name of table to insert rows to
    kwargs: should be of form COLUMN_NAME:[list, of, values, in, column]

    executemany takes two arguments:
        -1) the command string, with VALUES (%s, %s...%s) for #columns to write
        -2) a list of tuples of rows, e.g.
            data = [
                (row1_column1, row1_column2, ...row1_columnn),
                ...
                (rowm_column1, rowm_column2, ...rowm_columnn)
            ]
    """
    if connection is not None:
        conn = connection
    else:
        conn = qf.database_connection()

    cur = conn.cursor()
    column_names = "`" + "`, `".join(kwargs.keys()) + "`"
    values_sideways = array([v for v in kwargs.values()])
    values = values_sideways.T.tolist()
    # values = "'"+"', '".join([str(v) for v in kwargs.values()])+"'"
    for value in values:
        try:
            command = "INSERT INTO `%s` (%s) VALUES (%s)" % (
                table_name,
                column_names,
                str(value)[1:-1],
            )
            cur.execute(command)
            conn.commit()
            time.sleep(0.5)
        except mariadb.IntegrityError:
            pass
    if connection is None:
        conn.close()


def read_ice_log(path, date=False):
    log_columns = {
        "date": [],
        "time": [],
        "epochtime": [],
        "date_time": [],
        "T1": [],
        "T2": [],
        "T3": [],
        "T4": [],
        "sensor1": [],
        "sensor2": [],
        "sensor3": [],
        "sensor4": [],
        "heater1": [],
        "heater2": [],
        "needlevalve": [],
        "needlevalve2": [],
        "null1": [],
        "null2": [],
        "setpoint2": [],
        "20_1": [],
        "ramprate2": [],
        "null3": [],
        "P": [],
        "I": [],
        "D": [],
        "50_1": [],
        "20_2": [],
        "null4": [],
        "null5": [],
        "dump_pressure": [],
    }

    datetime_format = "%m/%d/%Y %I:%M:%S %p"

    if not date:
        # yesterday = datetime.now() - timedelta(1)
        date = datetime.strftime(datetime.now(), "%Y-%m-%d")

    path = path + date + "/"

    try:
        with open(path + date + ".log") as csvfile:
            read = csv.reader(csvfile)

            with open(path + "temp.log", "w", newline="\n") as csvfile:
                write = csv.writer(csvfile, delimiter=",")

                for i, row in enumerate(read):
                    log_columns["epochtime"].append(int(row[2]))
                    datetime_data = datetime.strftime(
                        datetime.strptime(row[3], datetime_format), "%Y-%m-%d %H:%M:%S"
                    )
                    log_columns["date_time"].append(datetime_data)
                    log_columns["T1"].append(float(row[4]))
                    log_columns["T2"].append(float(row[5]))
                    log_columns["T3"].append(float(row[6]))
                    log_columns["T4"].append(float(row[7]))
                    log_columns["needlevalve"].append(float(row[14]))
                    log_columns["dump_pressure"].append(float(row[29]))

                    write.writerow(
                        [
                            log_columns["epochtime"][i],
                            log_columns["date_time"][i],
                            log_columns["T1"][i],
                            log_columns["T2"][i],
                            log_columns["T3"][i],
                            log_columns["T4"][i],
                            log_columns["needlevalve"][i],
                            log_columns["dump_pressure"][i],
                        ]
                    )
    except Exception as e:
        print(e)
        time.sleep(600)

    load_data_to_database(path + "temp.log", "ice_log")
    file_to_rem = Path(path + "temp.log")
    file_to_rem.unlink()
    return


# schedule.every().day.at('01:00').do(read_ice_log, path=PATH)
# schedule.every(10).minutes.do(read_ice_log, path=PATH)

while True:
    read_ice_log(PATH)
    time.sleep(120)



# -*- coding: utf-8 -*-
"""
Created on Wed Mar  9 16:12:41 2022

@author: omedeiro
"""

from enum import Enum

import pyvisa


class PowerUnit(Enum):
    DBM = 0
    WATT = 1


class N7752A(object):
    """
    Python class for Keysight N7752A Optical Attenuator, modfied from JDS by Emma Batson.

    Original python class for JDS HJA9 Optical Attenuator, written by Adam McCaughan."""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands
        self.attenuation = 0
        self.set_beam_block(beam_block=False)

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def read_power(self, channel=1):
        # returns around -6e1 or -7e1 with laser off
        return self.query("READ{}:POW?".format(channel))

    def get_attenuation(self, channel=1):
        return self.query("INP{}:ATT?".format(channel))

    def get_output_power(self, channel=1):
        return self.query("OUTP{}:POW?".format(channel))

    def get_beam_block(self, channel=1):
        return self.query("OUTP{}:STAT?".format(channel))

    def toggle_power_control(self, pow_control=False, channel=1):
        self.write("OUTP{}:POW:CONTR {}".format(channel, int(pow_control)))

    # def set_attenuation_db(self, attenuation_db = 10, channel=1):
    #     # hlarocqu: Each channel can' go above 45 dB in attenuation it seems
    #     self.write('INP{}:ATT {}'.format(channel, attenuation_db))
    #     self.attenuation = attenuation_db

    # set_attenuation_db function for 0-90 dB range with cascaded attenuators
    def set_attenuation_db(self, attenuation_db=10):
        # hlarocqu: Each channel can' go above 45 dB in attenuation it seems
        if attenuation_db >= 90:
            print("Attenuation is capped at <90dB")
        attenuation_db = min(attenuation_db, 89.999)

        attenuation_db_1 = attenuation_db % 45
        # print(attenuation_db_1)
        attenuation_db_3 = (attenuation_db // 45) * 45
        # print(attenuation_db_3)

        self.write("INP{}:ATT {}".format(1, attenuation_db_1))
        self.write("INP{}:ATT {}".format(3, attenuation_db_3))
        self.attenuation = attenuation_db

    def set_beam_block(self, beam_block=True):
        self.write("OUTP{}:STAT {}".format(1, int(not beam_block)))
        self.write("OUTP{}:STAT {}".format(3, int(not beam_block)))

    # Additional Functions from hlarocqu

    # def output_on(self, channel=1):
    #     self.write('OUTP{} ON'.format(channel))

    # def output_off(self, channel=1):
    #     self.write('OUTP{} OFF'.format(channel))


# att = JDSHA9('GPIB0::10')



# -*- coding: utf-8 -*-
"""
Created on Fri Jul 31 09:31:52 2020

@author: omedeiro

This script is designed to extract the temperature data from ICE oxford
generated log files. Previous IoT efforts failed due to high memory consumption
and difficulity interfacing (caused by ICE software) with the temperature
controller


"""

import datetime
import os
import sys

import numpy as np
import scipy.io
from matplotlib import pyplot as plt

path = r"S:\SC\InstrumentLogging\Cryogenics\Ice\_copy_of_ice_log\Results"


# %%
""" Plot all TIME-TEMP data"""


def date_plot(path, date_start, date_end=datetime.date.today().isoformat(), plot=None):
    sys.path.append(path)
    folder_list = os.listdir(path)
    folder_list.sort()

    ds = datetime.datetime.strptime(date_start, "%Y-%m-%d")
    de = datetime.datetime.strptime(date_end, "%Y-%m-%d")

    """ Convert strings to dates for boolean condition """
    date_list = [
        datetime.datetime.strptime(folder_list[i], "%Y-%m-%d")
        for i in range(len(folder_list))
        if datetime.datetime.strptime(folder_list[i], "%Y-%m-%d") > ds
        and datetime.datetime.strptime(folder_list[i], "%Y-%m-%d") < de
    ]

    """ Convert dates back to strings """
    date_list_str = [d.strftime("%Y-%m-%d") for d in date_list]

    x = []
    y = []
    for r in date_list_str:
        new_path = os.path.join(path, r, r + ".log")
        f = open(new_path)
        lines = f.readlines()
        for line in lines:
            cells = line.split(",")
            date = datetime.datetime.strptime(cells[3], "%m/%d/%Y %I:%M:%S %p")
            data = [float(cells[4]), float(cells[5]), float(cells[6]), float(cells[7])]

            if data[0] < 290:
                x.append(date)
                y.append(data)
    if plot:
        plt.plot(x, y, marker="o", ls="None")
        plt.legend(["Sensor 1", "Sensor 2", "Sensor 3", "Sensor 4"], loc="upper right")

    return x, y


# %%
"""nice single line command """
# time_diff = [td.total_seconds() for td in np.diff(x) if td.total_seconds()>30]


def save_cycle(x, y):
    """
    This function takes in datetime.datetime and temperature data (generally
    created using date_plot()) and extracts each complete cooldown/warmup cycle.
    """

    """ Extract START-END points of temperature sweeps """
    time_diff_index = []
    dx = np.diff(x)
    for t in range(len(np.diff(x))):
        td = dx[t].total_seconds()
        if td > 10000:  # finds when the log was off.
            time_diff_index.append(t)
            print(x[t])

    """ Filter lists such that each sweep has High temp and Low temp"""
    trace_list = []
    for i in range(len(time_diff_index) - 1):
        temp = y[time_diff_index[i] + 1 : time_diff_index[i + 1] + 1]
        temp_array = np.asarray(temp, dtype=np.float64)
        if (
            temp_array[0, 0] > 150
            and temp_array[-1, 0] > 150
            and np.min(temp_array[:, 0]) < 5
        ):  # first and last points are high temp
            trace_list.append(i)

    """ Plot Sweeps """
    for i in trace_list:
        time = np.asarray(
            x[time_diff_index[i] + 1 : time_diff_index[i + 1]], dtype=np.datetime64
        )
        temp = np.asarray(
            y[time_diff_index[i] + 1 : time_diff_index[i + 1]], dtype=np.float64
        )
        time_str = str(time[0])[0:10]
        network_path = r"S:\SC\InstrumentLogging\Cryogenics\Ice"
        file_name = os.path.join(network_path, time_str)
        data_dict = {"time": time, "temp": temp}
        scipy.io.savemat(file_name + "_cooldown.mat", mdict=data_dict)
        print("Saving %s" % (time_str))
        plt.plot(time, temp)
        plt.pause(2)
        plt.close()


# get_start_end(x,   y)

# %%
x, y = date_plot(path, "2021-03-30", "2021-04-03", plot=True)
save_cycle(x, y)

# %%


""" Things to do 


then extract data parameters such as, cooldown/wawrmup time, min temp. etc




"""
